@@ -23,7 +23,9 @@ _DATA_TYPES = {
     'Image': [str],
     'List': [list],
     'Dialog': [list],
-    'Table': [dict]
+    'Table': [dict],
+    'TimeSeries': [dict, list, str],
+    'TimeSeriesChannel': [dict, list, str]
 }
 logger = logging.getLogger(__name__)
 settings = Settings()
@@ -84,9 +86,9 @@ class TaskValidator:
             TaskValidator.check_data(project, data)
         except ValidationError as e:
             if dict_is_root:
-                raise ValidationError(e.detail[0] + ' [assume: item = {...}, item is task root] ')
+                raise ValidationError(e.detail[0] + ' [assume: item as is = task root with values] ')
             else:
-                raise ValidationError(e.detail[0] + ' [assume: item = {"data": {...}}, item["data"] is task root]')
+                raise ValidationError(e.detail[0] + ' [assume: item["data"] = task root with values]')
 
     def project(self):
         """ Take the project from context
